library(loadeR)
library(transformeR)
library(ggplot2)
library(RColorBrewer)
library(maps)

rm(list=ls()) ; graphics.off() ; gc()

#Directory information
setwd("~/Documents/ETCCDI_eobs/")
OUTPUTS <- "OUTPUTS/"

#Useful information
ETCCDI_index <- c("Rx1day","Rx5day","TN10p","TN90p","TNn","TNx","TX10p","TX90p","TXn","TXx","CDD","CWD","CSDI","WSDI")
seasons <- list(Summer=c(6,7,8),Autumn=c(9,10,11),Winter=c(12,1,2),Spring=c(3,4,5))

calendario <- seq(as.Date("1950-01-01"),as.Date("2021-12-31"),by="day")
fechas <- calendario[format(calendario,"%m-%d") != "02-29"]

#Load CPs information
load("../WardCluster_Dendrogram_D2_50km.Rdata")
k=5
vector_cluster <- cutree(ward_clustering,k)[which(format(calendario,"%Y")%in%c(1950:2021))]
cp <- loadGridData(dataset = "../eobs_tasmax_spain.nc",lonLim = c(-1,1),latLim = c(35,36),var="tasmax")
cp <- aggregateGrid(cp,aggr.spatial = list(FUN="mean"))
for(l in 1:length(cp$Data)){
  cp$Data[l] <- vector_cluster[l]
}
value <- NA ; value.cat <- NA

for(ind in 1:length(ETCCDI_index)){
  #Load data
  files <- list.files(pattern = ETCCDI_index[ind])
  data <- list() ; data.cat <- list()
  
  for(i in 1:length(files)){
    di <- dataInventory(dataset = files[i])
    nc <- loadGridData(dataset=files[i],var=names(di))
    
    #Aggregation of annual/seasonal indices (climatology: time-series)
    if(length(nc$Dates$start)<=72){
      index2 <- aggregateGrid(nc,aggr.spatial = list(FUN="mean",na.rm=T))
      index.cat <- aggregateGrid(subsetGrid(nc,lonLim = c(0,3),latLim = c(40,43)),
                                 aggr.spatial = list(FUN="mean",na.rm=T))
  
      #Generating time-series for each CP
      for(l in 1:length(unique(vector_cluster))){
        v <- unique(vector_cluster)[l]
        cp2 <- cp
        cp2$Data[which(cp2$Data!=v)] <- 0
        cp2$Data[which(cp2$Data==v)] <- 1
        index.cp <- aggregateGrid(cp2,aggr.s = list(FUN=list("sum",na.rm=T),season=c(1:12)))
        value[l] <- cor(x=index.cp$Data,index2$Data,use = "pairwise.complete.obs",method = "spearman")
        value.cat[l] <- cor(x=index.cp$Data,index.cat$Data,use = "pairwise.complete.obs",method = "spearman")
      }
      
      data[[1]] <- data.frame(Index=value,
                              Season=rep("Annual",length(value)),
                              CP=unique(vector_cluster))
      data.cat[[1]] <- data.frame(Index=value.cat,
                                    Season=rep("Annual",length(value)),
                                    CP=unique(vector_cluster))
     }
    if(length(nc$Dates$start)>72){
      for(j in 1:length(seasons)){
        index2 <- redim(aggregateGrid(nc,aggr.s = list(FUN=list("mean",na.rm=T),season=seasons[[j]])),drop=T)
        index.cat <- subsetGrid(index2,lonLim = c(0,3),latLim = c(40,43))
        
        index2 <- aggregateGrid(index2,aggr.spatial = list(FUN="mean",na.rm=T))
        index.cat <- aggregateGrid(index.cat,aggr.spatial = list(FUN="mean",na.rm=T))
        
        #Generating time-series for each CP
        for(l in 1:length(unique(vector_cluster))){
          v <- unique(vector_cluster)[l]
          cp2 <- cp
          cp2$Data[which(cp2$Data!=v)] <- 0
          cp2$Data[which(cp2$Data==v)] <- 1
        index.cp <- aggregateGrid(cp2,aggr.s = list(FUN=list("sum",na.rm=T),season=seasons[[j]]))
        value[l] <- cor(x=index.cp$Data,index2$Data,use = "pairwise.complete.obs",method = "spearman")
        value.cat[l] <- cor(x=index.cp$Data,index.cat$Data,use = "pairwise.complete.obs",method = "spearman")
        }
        
        data[[j+1]] <- data.frame(Index=value,
                                Season=rep(names(seasons)[j],length(value)),
                                CP=unique(vector_cluster))
        data.cat[[j+1]] <- data.frame(Index=value.cat,
                                    Season=rep(names(seasons)[j],length(value)),
                                    CP=unique(vector_cluster))
      }}
  }
  
  #Organize for ggplot plotting and saving
  datos <- do.call(rbind,data)
  datos$Season <- factor(datos$Season,levels=c("Winter","Autumn","Summer","Spring","Annual"))
  
  datos.cat <- do.call(rbind,data.cat)
  datos.cat$Season <- factor(datos.cat$Season,levels=c("Winter","Autumn","Summer","Spring","Annual"))
  
  mix <- rbind(datos,datos.cat)
  mix$Region <- rep(c("Iberian Peninsula","Catalunya"),each=dim(datos)[1])
  
  png(filename = paste(OUTPUTS,ETCCDI_index[ind],"_E-OBS_Correlations_D2k1.png",sep=""),res=300,width=1500,height=1500)
  print(ggplot(data=mix,aes(x=Season,y=CP,fill=Index)) + geom_tile() + facet_wrap(~Region,nrow = 3) +
          theme_bw() + ggtitle(paste("E-OBS",names(di),sep=" ")) + 
          scale_color_gradientn("Correlation",colours = rev(brewer.pal(n=11,name="RdBu")),limits=c(-1,1),
                                aesthetics = "fill",na.value = "white") + geom_text(aes(label=round(Index,digits=2)))) 
  dev.off()
} 
